/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2016                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_off.h>
#include<wrap/io_trimesh/export_obj.h>
#include<wrap/io_trimesh/export_ply.h>


class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::Color4b, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};
using namespace vcg;

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.obj>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;

  if(tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=vcg::tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }
  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());  
  tri::UpdateTopology<MyMesh>::FaceFace(m);
  tri::UpdateFlags<MyMesh>::FaceClearV(m);
  MyMesh::PerFaceAttributeHandle<float> h_FQ = 
      tri::Allocator<MyMesh>::GetPerFaceAttribute<float> (m);
  
  std::stack<MyMesh::FacePointer> visitStack;
  int componentNum=0;
  int faceCnt=0;
  for(auto fi=m.face.begin();fi!=m.face.end();++fi)
    if(!fi->IsV())
    {
      ++componentNum;
      visitStack.push(&*fi);
      int visitedNum=0;
      while(!visitStack.empty())
      {
        MyMesh::FacePointer curF = visitStack.top(); visitStack.pop();
        if(!curF->IsV()) 
        {
          ++visitedNum;
          curF->SetV();
          ++faceCnt;
          //          curF->C()=Color4b::ColorRamp(0,m.FN(),faceCnt);
          float perc = float(faceCnt)/float(m.FN());
          h_FQ[curF] = perc;
          curF->C()=Color4b::ColorRamp(0,m.FN(),faceCnt);
          printf("face %lu gray %f %i %i %i \n", tri::Index(m,curF),perc,
                 curF->C()[0],curF->C()[1],curF->C()[2]);
          
          for(int i=0;i<3;++i)
            if(!curF->FFp(i)->IsV())
              visitStack.push(curF->FFp(i));      
        }   
      }
      printf("Visited face Number %i\n",visitedNum);
    }
  printf("Component Number %i\n",componentNum);
  tri::io::ExporterPLY<MyMesh>::Save(m,"color.ply",tri::io::Mask::IOM_FACECOLOR + tri::io::Mask::IOM_FACEQUALITY);
  
  return 0;
}
